`timescale 1ns / 1ps
//////////////////////////////////////////////////////////////////////////////////
// Company: 
// Engineer: 
// 
// Create Date:    13:21:36 02/10/2020 
// Design Name: 
// Module Name:    adda_test 
// Project Name: 
// Target Devices: 
// Tool versions: 
// Description: 
//
// Dependencies: 
//
// Revision: 
// Revision 0.01 - File Created
// Additional Comments: 
//
//////////////////////////////////////////////////////////////////////////////////
module DDS(
    input 			clk     ,                 //fpga clock
	 input 			rst_n   ,
	 input  [3:0] 	key_done,
    output [7:0] 	dadata        
    );
	 

reg  [7:0]  i        ;
reg  [8:0]  phase    ;
reg  [8:0]  rom_addr ;
reg  [7:0]  dadata_r ;
reg  [1:0]  wave_cnt ;
wire        add_i;
wire        end_i;
wire        add_phase;
wire        end_phase;
wire        add_wave_cnt;
wire        end_wave_cnt;

wire [7:0] sin_data;
wire [7:0] triangle_data;
wire [7:0] sawtooth_data;
wire [7:0] wave_data;

always @(posedge clk or negedge rst_n)        
begin
   if(!rst_n)begin
		i <= 4'd1;
	end
	else if(add_i)begin
		if(end_i)
			i <= 4'd1;
		else 
			i <= 2*i;
	end
	else 
		i <= i;
end
assign add_i = key_done[1];
assign end_i = add_i && i==64;

always @(posedge clk or negedge rst_n)        
begin
   if(!rst_n)begin
		phase <= 9'd0;
	end
	else if(add_phase)begin
		if(end_phase)
			phase <= 9'd0;
		else 
			phase <= phase + 9'd30;
	end
	else 
		phase <= phase;
end
assign add_phase = key_done[2];
assign end_phase = add_phase && phase==360;

always@(posedge clk or negedge rst_n)
begin
	if(!rst_n)
		rom_addr <= 9'd0;
	else
		rom_addr <= phase + i;
end
	

always @ (posedge clk or negedge rst_n)      //
begin
	if(!rst_n)begin                              //
		wave_cnt <= 2'd0;
	end
   else if(add_wave_cnt)begin
		if(end_wave_cnt)
			wave_cnt <= 2'd0;
		else 
			wave_cnt <= wave_cnt + 1;
	end
	else 
		wave_cnt <= wave_cnt;
end
assign add_wave_cnt = key_done[0];
assign end_wave_cnt = add_wave_cnt && wave_cnt==4-1;

always@(*)begin
	case(wave_cnt)
		  0 :  dadata_r <= sin_data;
		  1 :  dadata_r <= triangle_data;
		  2 :  dadata_r <= sawtooth_data;
   default:  dadata_r <= sin_data;
	endcase
end
assign dadata = dadata_r;




//例化正弦波ROM
ROM_SIN 			ROM_SIN_Init
(
	.address		(rom_addr		),	//ROM地址
	.clock		(clk				),	//ROM时钟
	.q				(sin_data		)	//从ROM中读出的正弦波数据
);

//例化三角波ROM
ROM_TRIANGLE 	ROM_TRIANGLE_Init
(
	.address		(rom_addr		), //ROM地址
	.clock		(clk     		), //ROM时钟
	.q				(triangle_data	)  //从ROM中读出的三角波数据
);

//例化锯齿波ROM
ROM_SAWTOOTH 	ROM_SAWTOOTH_Init
(
	.address		(rom_addr		), //ROM地址
	.clock		(clk     		), //ROM时钟
	.q				(sawtooth_data	)  //从ROM中读出的锯齿波数据
);

endmodule

